#include "bootstrap.h"
#include "kernel.h"
#include "omp.h"

ok_list* K_bootstrap(ok_kernel* k, int trials, int warmup, int malgo, int miter, double mparams[]) {
    ok_list* kl = KL_alloc(trials);
    ok_list* wu = NULL;
    ok_progress prog = k->progress;
    
    int nthreads = omp_get_max_threads();
    bool invalid = false;
    
    if (warmup > 0) {
        wu = KL_alloc(warmup);
        #pragma omp parallel for

        for (int i = 0; i < warmup; i++) {
            if (invalid)
                continue;
            ok_kernel* k2 = K_clone(k);
            k2->flags |= NEEDS_COMPILE | BOOTSTRAP_DATA;
            k2->progress = NULL;
            K_minimize(k2, malgo, miter, mparams);
            wu->kernels[i] = k2;
            if (prog != NULL && omp_get_thread_num() == 0) {
                int ret = prog(i, (int)((double) warmup / (double) nthreads), k2,
                    "K_bootstrap_warmup");
                if (ret == PROGRESS_STOP) {
                    invalid = true;
                }
            }
        }
    }
    
    
    
    #pragma omp parallel for
    for (int i = 0; i < trials; i++) {
        if (invalid)
            continue;
        
        ok_kernel * k2 = K_cloneFlags(k, SHARE_FLAGS | SHARE_STEPS);
        k2->flags |= NEEDS_COMPILE | BOOTSTRAP_DATA;
        k2->progress = NULL;
        K_calculate(k2);
        K_minimize(k2, malgo, miter, mparams);
        kl->kernels[i] = k2;
        
        if (prog != NULL && omp_get_thread_num() == 0) {
            int ret = prog(i, (int)((double) trials / (double) nthreads), k2,
                    "K_bootstrap");
            if (ret == PROGRESS_STOP) {
                invalid = true;
            }
        }
    }
    
    if (invalid) {
        KL_free(wu);
        KL_free(kl);
        return NULL;
    }
        
    
    KL_free(wu);
    return kl;
}
